%options package=com.ibm.adt.ovf.ext.qualifier
%options fp=QualifierLexer
%options template=LexerTemplateD.g
%options la=1
%options verbose


%Globals
    /.import java.util.*;
    ./
%End

%Define 
--    $kw_lexer_class /.Keyword./
      $super_stream_class /.QualifierLexStream./
%End

%Include 
    LexerVeryBasicMap.g
%End

%Export
        IDENTIFIER 
        PLUS
        MINUS
        LEFTPAREN
        RIGHTPAREN
        DOT
        LBRACE
        RBRACE
        LBRACKET
        RBRACKET
        COMMA
        BINARYVALUE
        OCTALVALUE
        HEXVALUE
        DECIMALVALUE
        REALVALUE
        CHARVALUE
        STRINGFRAG
        TRUE
        FALSE
        NULL
%End

%Terminals
    CtlCharNotWS

    LF   CR   HT   FF

    a    b    c    d    e    f    g    h    i    j    k    l    m
    n    o    p    q    r    s    t    u    v    w    x    y    z
    _

    A    B    C    D    E    F    G    H    I    J    K    L    M
    N    O    P    Q    R    S    T    U    V    W    X    Y    Z

    0    1    2    3    4    5    6    7    8    9

    AfterASCII   ::= '\u0080..\ufffe'
    Space        ::= ' '
    LF           ::= NewLine
    CR           ::= Return
    HT           ::= HorizontalTab
    FF           ::= FormFeed
    DoubleQuote  ::= '"'
    SingleQuote  ::= "'"
    Percent      ::= '%'
    VerticalBar  ::= '|'
    Exclamation  ::= '!'
    AtSign       ::= '@'
    BackQuote    ::= '`'
    Tilde        ::= '~'
    Sharp        ::= '#'
    DollarSign   ::= '$'
    Ampersand    ::= '&'
    Caret        ::= '^'
    Colon        ::= ':'
    SemiColon    ::= ';'
    BackSlash    ::= '\'
    LeftBrace    ::= '{'
    RightBrace   ::= '}'
    LeftBracket  ::= '['
    RightBracket ::= ']'
    QuestionMark ::= '?'
    Comma        ::= ','
    Dot          ::= '.'
    LessThan     ::= '<'
    GreaterThan  ::= '>'
    Plus         ::= '+'
    Minus        ::= '-'
    Slash        ::= '/'
    Star         ::= '*'
    LeftParen    ::= '('
    RightParen   ::= ')'
    Equal        ::= '='
%End

%Start
    Token
%End

%Rules
    Token ::= 
            | '+'           /.    makeToken($_PLUS);./
            | '-'           /.    makeToken($_MINUS);./
            | '('           /.    makeToken($_LEFTPAREN);./
            | ')'           /.    makeToken($_RIGHTPAREN);./
            | '.'           /.    makeToken($_DOT);./
            | ' '           /.    skipToken();./
            | '{'           /.    makeToken($_LBRACE);./
            | '}'           /.    makeToken($_RBRACE);./
            | '['           /.    makeToken($_LBRACKET);./
            | ']'           /.    makeToken($_RBRACKET);./
            | ','           /.    makeToken($_COMMA);./
            | OTHERWS       /.    skipToken();./
            | BinaryValue   /.    if(checkBinaryValue(getLeftSpan(), getRightSpan())) { makeToken($_BINARYVALUE);}./
            | OctalValue    /.    makeToken($_OCTALVALUE);./
            | HexValue      /.    makeToken($_HEXVALUE);./
            | DecimalValue  /.    makeToken($_DECIMALVALUE);./
            | RealValue     /.    makeToken($_REALVALUE);./
            | CharValue     /.    makeToken($_CHARVALUE);./
            | StringFrag    /.    makeToken($_STRINGFRAG);./

    Token ::= identifier
       /.
              String s = new String(getInputChars(), getLeftSpan(), getRightSpan() - getLeftSpan() + 1);
              if("true".equalsIgnoreCase(s)) {
                  makeToken($_TRUE);            	  
              } else if("false".equalsIgnoreCase(s)) {
                  makeToken($_FALSE);            	  
              } else if("null".equalsIgnoreCase(s)) {
                  makeToken($_NULL);            	  
              } else {
                  makeToken($_IDENTIFIER);            	  
              }
      ./
                
    firstIdChar -> letter | _ | AfterASCII

    idChar ::= firstIdChar | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | Minus
    
    identifier -> firstIdChar
                | identifier idChar

    BinaryValue ::= -- 1 BinaryDigits Bb
                   DecimalValue Bb -- NOTE: This binary digits must be verified by a semantic check
    
    Bb ::= B | b
    
    OctalValue ::= 0 OctalDigits
    
    OctalDigits ::= OctalDigit
                    | OctalDigits OctalDigit
                    
    OctalDigit ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7

    DecimalValue ::= PositiveDecimalDigit DecimalDigits
                     | 0
    
    DecimalDigits ::= %empty
                      | DecimalDigits DecimalDigit
                      
    PositiveDecimalDigit ::= 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
    
    DecimalDigit ::= OctalDigit | 8 | 9
    
    HexValue ::= 0 Xx HexDigits
    
    Xx ::= x | X
    
    HexDigits ::= HexDigit 
                  | HexDigits HexDigit

    HexDigit ::= DecimalDigit | a | A | b | B | c | C | e | E | f | F
                   
    RealValue ::= DecimalValue Dot DecimalValue ExponentTerm
    
    ExponentTerm ::= %empty
                     | Ee OptionalSign DecimalValue
                     
    Ee ::= e | E
    
    OptionalSign ::= %empty | Minus | Plus
    
    CharValue ::= SingleQuote  NotSQ SingleQuote
    
    NotSQ ::= 
        letter | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |
        AfterASCII   | 
        Space        | 
        LF           | 
        CR           | 
        HT           | 
        FF           | 
        DoubleQuote  | 
        Percent      | 
        VerticalBar  | 
        Exclamation  | 
        AtSign       | 
        BackQuote    | 
        Tilde        | 
        Sharp        | 
        DollarSign   | 
        Ampersand    | 
        Caret        | 
        Colon        | 
        SemiColon    | 
        BackSlash    | 
        LeftBrace    | 
        RightBrace   | 
        LeftBracket  | 
        RightBracket | 
        QuestionMark | 
        Comma        | 
        Dot          | 
        LessThan     | 
        GreaterThan  | 
        Plus         | 
        Minus        | 
        Slash        | 
        Star         | 
        LeftParen    | 
        RightParen   | 
        Equal         
        
    StringFrag ::= DoubleQuote StringBody DoubleQuote
    
    StringBody ::= %empty | StringBody NotDQ
    
    NotDQ ::=
        letter | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |
        AfterASCII   | 
        Space        | 
        LF           | 
        CR           | 
        HT           | 
        FF           | 
        SingleQuote  | 
        Percent      | 
        VerticalBar  | 
        Exclamation  | 
        AtSign       | 
        BackQuote    | 
        Tilde        | 
        Sharp        | 
        DollarSign   | 
        Ampersand    | 
        Caret        | 
        Colon        | 
        SemiColon    | 
        LeftBrace    | 
        RightBrace   | 
        LeftBracket  | 
        RightBracket | 
        QuestionMark | 
        Comma        | 
        Dot          | 
        LessThan     | 
        GreaterThan  | 
        Plus         | 
        Minus        | 
        Slash        | 
        Star         | 
        LeftParen    | 
        RightParen   | 
        Equal        |
        BackSlash BackSlash |
        BackSlash DoubleQuote
        
    aA ::= a | A
    bB ::= b | B
    cC ::= c | C
    dD ::= d | D
    eE ::= e | E
    fF ::= f | F
    gG ::= g | G
    hH ::= h | H
    iI ::= i | I
    jJ ::= j | J
    kK ::= k | K
    lL ::= l | L
    mM ::= m | M
    nN ::= n | N
    oO ::= o | O
    pP ::= p | P
    qQ ::= q | Q
    rR ::= r | R
    sS ::= s | S
    tT ::= t | T
    uU ::= u | U
    vV ::= v | V
    wW ::= w | W
    xX ::= x | X
    yY ::= y | Y
    zZ ::= z | Z

    letter ::= aA | bB | cC | dD | eE | fF | gG | hH | iI | jJ | kK | lL | mM | nN | oO | pP | qQ | rR | sS | tT | uU | vV | wW | xX | yY | zZ

    OTHERWS ::= LF | CR | HT 

%End
