/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import org.dmtf.schemas.ovf.envelope.EnvelopeFactory;
import org.dmtf.schemas.ovf.envelope.FileType;
import org.dmtf.schemas.ovf.envelope.ReferencesType;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.util.EContentAdapter;

public class AdapterExample {

	public static void main(String[] args) {
		AdapterExample example = new AdapterExample();
		example.run(args);
	}

	private void run(String[] args) {
		System.out.println("Example 1:");
		example1();
		System.out.println("Example 2:");
		example2();
	}

	private void example1() {
		ReferencesType refs = EnvelopeFactory.eINSTANCE.createReferencesType();
		MyAdapter refsAdapter = new MyAdapter("refs adapter");
		refs.eAdapters().add(refsAdapter);
		System.out.println("Added refs adapter");

		FileType file1 = EnvelopeFactory.eINSTANCE.createFileType();
		MyAdapter file1Adapter = new MyAdapter("file1 adapter");
		file1.eAdapters().add(file1Adapter);
		System.out.println("Added file1 adapter");

		refs.getFile().add(file1); // fires ADD event

		file1.setId("file1"); // fires SET event
		file1.getId(); // no event fired

		refs.getFile().clear(); // fires REMOVE event for every FileType removed

		// Clean up by removing adapters
		refs.eAdapters().remove(refsAdapter); // fires REMOVING_ADAPTER
		file1.eAdapters().remove(file1Adapter); // fires REMOVING_ADAPTER		
	}

	private void example2() {
		ReferencesType refs = EnvelopeFactory.eINSTANCE.createReferencesType();
		MyContentAdapter refsAdapter = new MyContentAdapter("refs content adapter");
		refs.eAdapters().add(refsAdapter);
		System.out.println("Added refs content adapter");

		FileType file1 = EnvelopeFactory.eINSTANCE.createFileType();

		refs.getFile().add(file1); // fires ADD event

		file1.setId("file1"); // fires SET event

		refs.getFile().clear(); // fires SET event for setting list to null

		// Clean up by removing adapters
		refs.eAdapters().remove(refsAdapter); // fires REMOVING_ADAPTER
	}
	
	class MyAdapter extends AdapterImpl {

		String name;

		public MyAdapter(String name) {
			this.name = name;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.
		 * eclipse.emf.common.notify.Notification)
		 */
		@Override
		public void notifyChanged(Notification msg) {
			super.notifyChanged(msg);
			System.out.println(name + ": " + getEventTypeName(msg));
		}

		private String getEventTypeName(Notification msg) {
			String eventType;
			switch (msg.getEventType()) {
			case Notification.SET: {
				eventType = "SET";
				break;
			}
			case Notification.UNSET: {
				eventType = "UNSET";
				break;
			}
			case Notification.ADD: {
				eventType = "ADD";
				break;
			}
			case Notification.ADD_MANY: {
				eventType = "ADD_MANY";
				break;
			}
			case Notification.REMOVE: {
				eventType = "REMOVE";
				break;
			}
			case Notification.REMOVE_MANY: {
				eventType = "REMOVE_MANY";
				break;
			}
			case Notification.MOVE: {
				eventType = "MOVE";
				break;
			}
			case Notification.REMOVING_ADAPTER: {
				eventType = "REMOVING_ADAPTER";
				break;
			}
			case Notification.RESOLVE: {
				eventType = "RESOLVE";
				break;
			}
			default: {
				eventType = "Unknown";
				break;
			}
			}
			return eventType;
		}
	}

	class MyContentAdapter extends EContentAdapter {

		String name;

		public MyContentAdapter(String name) {
			this.name = name;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.
		 * eclipse.emf.common.notify.Notification)
		 */
		@Override
		public void notifyChanged(Notification msg) {
			super.notifyChanged(msg);
			System.out.println(name + ": " + getEventTypeName(msg) + " for "+msg.getNotifier());
		}

		private String getEventTypeName(Notification msg) {
			String eventType;
			switch (msg.getEventType()) {
			case Notification.SET: {
				eventType = "SET";
				break;
			}
			case Notification.UNSET: {
				eventType = "UNSET";
				break;
			}
			case Notification.ADD: {
				eventType = "ADD";
				break;
			}
			case Notification.ADD_MANY: {
				eventType = "ADD_MANY";
				break;
			}
			case Notification.REMOVE: {
				eventType = "REMOVE";
				break;
			}
			case Notification.REMOVE_MANY: {
				eventType = "REMOVE_MANY";
				break;
			}
			case Notification.MOVE: {
				eventType = "MOVE";
				break;
			}
			case Notification.REMOVING_ADAPTER: {
				eventType = "REMOVING_ADAPTER";
				break;
			}
			case Notification.RESOLVE: {
				eventType = "RESOLVE";
				break;
			}
			default: {
				eventType = "Unknown";
				break;
			}
			}
			return eventType;
		}
	}
}
