/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import org.dmtf.schemas.ovf.envelope.DiskSectionType;
import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopeFactory;
import org.dmtf.schemas.ovf.envelope.EnvelopePackage;
import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.ReferencesType;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeResourceFactoryImpl;
import org.dmtf.schemas.ovf.envelope.util.OVFLabelProvider;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EValidator.SubstitutionLabelProvider;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.Diagnostician;

/**
 * <!-- begin-user-doc -->
 * A sample utility for the '<em><b>envelope</b></em>' package.
 * <!-- end-user-doc -->
 * @generated
 */
public class EnvelopeExample {
	/**
	 * <!-- begin-user-doc -->
	 * Load all the argument file paths or URIs as instances of the model.
	 * <!-- end-user-doc -->
	 * @param args the file paths or URIs.
	 * @generated
	 */
	@SuppressWarnings("deprecation")
	public static void main(String[] args) {
		// Create a resource set to hold the resources.
		//
		// A ResourceSet is where EMF holds persisted objects. As the name suggests,
		// it is a set of EMF Resources. When you load an OVF file, it will represented
		// as a Resource in a ResourceSet.
		ResourceSet resourceSet = new ResourceSetImpl();
		
		// Register the appropriate resource factory to handle all file extensions.
		// Since we are running outside of eclipse, we have to tell EMF how to 
		// associate files with models (like OVF). The DEFAULT_EXTENSION is "*" so
		// EMF will associate our Envelope stuff with any file.
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
			(Resource.Factory.Registry.DEFAULT_EXTENSION, 
			 new EnvelopeResourceFactoryImpl());

		// Register the package to ensure it is available during loading.
		// Since we are running outside of eclipse, we need to register our OVF metamodel
		// with the nsURI that identifies it. For OVF, this is "http://schemas.dmtf.org/ovf/envelope/1".
		resourceSet.getPackageRegistry().put
			(EnvelopePackage.eNS_URI, 
			 EnvelopePackage.eINSTANCE);
        
		// If there are no arguments, emit an appropriate usage message.
		//
		if (args.length == 0) {
			System.out.println("Enter a list of file paths or URIs that have content like this:");
			try {
				// Create a default OVF descriptor
				
				// Create an empty resource in our resource set. The URI is used to identify
				// the location of the file to be saved or loaded. For standalone applications,
				// you can use URIs like file://.... For this example, we are going to save
				// the OVF file to the system out stream, so it doesn't matter what we use for the
				// URI.
				Resource resource = resourceSet.createResource(URI.createURI("http:///My.ovf"));
				
				// The root element for our OVF metamodel is DocumentRoot. This is a fictitious
				// element that serves as a container for various XML things that don't 
				// really belong to the real root element, EnvelopeType.
				// Use the EnvelopeFactory to create instances of OVF objects instead
				// of java constructors -- EMF needs to initialize internal stuff via
				// the factory classes.
				DocumentRoot documentRoot = EnvelopeFactory.eINSTANCE.createDocumentRoot();
				
				// The resource is empty. We need to add the root to the resource. Normally,
				// this will be the only contents for the resource. If we were to add other
				// objects to the resource, they would be saved to the file, which would not
				// be valid for OVF. All the objects that are contained within the root are
				// automatically part of the resource.
				resource.getContents().add(documentRoot);
				
				// The document root has no Envelope type yet
				EnvelopeType envelope = EnvelopeFactory.eINSTANCE.createEnvelopeType();
				documentRoot.setEnvelope(envelope);

				// The envelope so far does not have a References element, so let's create one and
				// set it.
				ReferencesType references = EnvelopeFactory.eINSTANCE.createReferencesType();
				envelope.setReferences(references);
				
				// The OVF schema defines Sections as substitution groups and as generic Section
				// types. The EnvelopeType.sectionGroup maps the generic to the substitutions
				
				DiskSectionType diskSection = EnvelopeFactory.eINSTANCE.createDiskSectionType();
				
				// The EnvelopeType.getSection() method returns a list of sections. We add new
				// sections to this list. Doing it this way causes the "loose type" form of output.
				envelope.getSection().add(diskSection);
				
				// We can use the setDiskSection method more directly, and this causes the "strong typing" form
				// to be output. Uncomment the following line to see the difference
				//envelope.setDiskSection(diskSection);

				// When we want to save the model, use the Resource.save() methods. For this example
				// model, we will just put it to stdout. 
				resource.save(System.out, null);
			}
			catch (IOException exception) {
				exception.printStackTrace();
			}
		}
		else {
			// Iterate over all the arguments.
			//
			for (int i = 0; i < args.length; ++i) {
				// Construct the URI for the instance file.
				// The argument is treated as a file path only if it denotes an existing file.
				// Otherwise, it's directly treated as a URL.
				//
				File file = new File(args[i]);
				
				// Get an EMF URI (which is a different class from the java.net.URI) from the arguments.
				URI uri = file.isFile() ? URI.createFileURI(file.getAbsolutePath()): URI.createURI(args[0]);

				try {
					// Demand load resource for this file. ResourceSet.getResource() will locate the resource
					// if it has already been loaded, or it will load it from the source specified by the URI.
					//
					Resource resource = resourceSet.getResource(uri, true);
					System.out.println("Loaded " + uri);

					// Validate the contents of the loaded resource.
					//
					
					// Use a substitution label provider to give better labels in error messages
					SubstitutionLabelProvider labelProvider = new OVFLabelProvider();
					Map<Object, Object> context = Diagnostician.INSTANCE.createDefaultContext();
					context.put(SubstitutionLabelProvider.class, labelProvider);
					for (EObject eObject : resource.getContents()) {
						// You can validate on demand using the Diagnostician 
						Diagnostic diagnostic = Diagnostician.INSTANCE.validate(eObject, context);
						if (diagnostic.getSeverity() != Diagnostic.OK) {
							printDiagnostic(diagnostic, "");
						}
					}
					
					// If we assume the resource corresponds to an OVF descriptor, we can get 
					// a reference to the DocumentRoot
					DocumentRoot documentRoot = (DocumentRoot) resource.getContents().get(0);
					
					// Get the envelope
					EnvelopeType envelope = documentRoot.getEnvelope();
					
					// Count the sections in the envelope
					System.out.println("Found "+envelope.getAllSections().size()+" sections");
					
					// If we have modified the model, we would save it back to the filesystem like this
					try {
						resource.save(Collections.EMPTY_MAP); // Save with no options
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				catch (RuntimeException exception) {
					System.out.println("Problem loading " + uri);
					exception.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * Prints diagnostics with indentation.
	 * <!-- end-user-doc -->
	 * @param diagnostic the diagnostic to print.
	 * @param indent the indentation for printing.
	 * @generated
	 */
	protected static void printDiagnostic(Diagnostic diagnostic, String indent) {
		System.out.print(indent);
		System.out.println(diagnostic.getMessage());
		for (Diagnostic child : diagnostic.getChildren()) {
			printDiagnostic(child, indent + "  ");
		}
	}

} //EnvelopeExample
