/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopePackage;
import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeResourceFactoryImpl;
import org.dmtf.schemas.ovf.environment.EnvironmentFactory;
import org.dmtf.schemas.ovf.environment.EnvironmentType;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

/**
 * <!-- begin-user-doc --> A sample utility for the '<em><b>envelope</b></em>'
 * package. <!-- end-user-doc -->
 * 
 * @generated
 */
public class EnvironmentExample {
	/**
	 * <!-- begin-user-doc --> Load all the argument file paths or URIs as
	 * instances of the model. <!-- end-user-doc -->
	 * 
	 * @param args
	 *            the file paths or URIs.
	 * @generated
	 */
	public static void main(String[] args) {
		// Create a resource set to hold the resources.
		//
		// A ResourceSet is where EMF holds persisted objects. As the name
		// suggests,
		// it is a set of EMF Resources. When you load an OVF file, it will
		// represented
		// as a Resource in a ResourceSet.
		ResourceSet resourceSet = new ResourceSetImpl();

		// Register the appropriate resource factory to handle all file
		// extensions.
		// Since we are running outside of eclipse, we have to tell EMF how to
		// associate files with models (like OVF). The DEFAULT_EXTENSION is "*"
		// so
		// EMF will associate our Envelope stuff with any file.
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION,
						new EnvelopeResourceFactoryImpl());

		// Register the package to ensure it is available during loading.
		// Since we are running outside of eclipse, we need to register our OVF
		// metamodel
		// with the nsURI that identifies it. For OVF, this is
		// "http://schemas.dmtf.org/ovf/envelope/1".
		resourceSet.getPackageRegistry().put(EnvelopePackage.eNS_URI,
				EnvelopePackage.eINSTANCE);
		if (args.length != 2) {
			System.out.println("Syntax: <input-ovf> <output-dir>");
			return;
		}

		// Construct the URI for the instance file.
		// The argument is treated as a file path only if it denotes an existing
		// file.
		// Otherwise, it's directly treated as a URL.
		//
		File file = new File(args[0]);
		File dir = new File(args[1]);

		// Get an EMF URI (which is a different class from the java.net.URI)
		// from the arguments.
		URI uri = file.isFile() ? URI.createFileURI(file.getAbsolutePath())
				: URI.createURI(args[0]);

		try {
			// Demand load resource for this file. ResourceSet.getResource()
			// will locate the resource
			// if it has already been loaded, or it will load it from the source
			// specified by the URI.
			//
			Resource resource = resourceSet.getResource(uri, true);
			System.out.println("Loaded " + uri);

			// If we assume the resource corresponds to an OVF descriptor, we
			// can get
			// a reference to the DocumentRoot
			DocumentRoot documentRoot = (DocumentRoot) resource.getContents()
					.get(0);

			// Get the envelope
			EnvelopeType envelope = documentRoot.getEnvelope();

			// You can create all the environments for the virtual systems at once
			// with EnvelopeType.createEnvironments()
			// You can also create them with VirtualSystemCollectionType.createEnvironments()
			// and VirtualSystemType.createEnvironment()
			List<EnvironmentType> envs = envelope.createEnvironments();
			for(EnvironmentType env : envs) {
				File outFile = new File(dir, "env-"+env.getId()+".xml");
				Resource res = resourceSet.createResource(URI.createURI(env.getId()));
				org.dmtf.schemas.ovf.environment.DocumentRoot envDocRoot = EnvironmentFactory.eINSTANCE.createDocumentRoot();
				envDocRoot.setEnvironment(env);
				try {
					res.getContents().add(envDocRoot);
					FileOutputStream fos = new FileOutputStream(outFile); 
					res.save(System.out, Collections.EMPTY_MAP);
					res.save(fos, Collections.EMPTY_MAP);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			System.out.println("\n\nMade " + envs.size() + " environments");

		} catch (RuntimeException exception) {
			System.out.println("Problem loading " + uri);
			exception.printStackTrace();
		}
	}
} // EnvelopeExample
