/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import java.io.File;

import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopePackage;
import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeResourceFactoryImpl;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import com.ibm.adt.ovf.model.utility.OverwriteNotAllowedException;
import com.ibm.adt.ovf.model.utility.PackageExporter;


public class ExportPackageExample {

	public static void main(String[] args) {
		ExportPackageExample example = new ExportPackageExample();
		example.run(args);
	}

	private void run(String[] args) {
		ResourceSet resourceSet = new ResourceSetImpl();
		
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
			(Resource.Factory.Registry.DEFAULT_EXTENSION, 
			 new EnvelopeResourceFactoryImpl());

		resourceSet.getPackageRegistry().put
			(EnvelopePackage.eNS_URI, 
			 EnvelopePackage.eINSTANCE);
        
		Resource resource = resourceSet.getResource(URI.createURI("exportExampleData/lamp.ovf"), true);		
		DocumentRoot documentRoot = (DocumentRoot) resource.getContents().get(0);
		EnvelopeType envelope = documentRoot.getEnvelope();
				
		PackageExporter exporter = new PackageExporter(envelope, 
				null, // packageName default to envelope's filename 
				null, // baseLocalRef not needed
				new File("exportExampleData/lamp.ovf"), // baseFile
				new File("exportOutput"), // directory for output
				false, // isOVA - false means use directory files, not archive
				true, // allowOverwrite 
				true, // hasDigests
				null, // private key
				null, // certificate
				null // progressMonitor
				);
		try {
			exporter.export();
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		// Do another export, but disallow overwriting
		exporter = new PackageExporter(envelope, 
				null, // packageName default to envelope's filename 
				null, // baseLocalRef not needed
				new File("exportExampleData/lamp.ovf"), // baseFile
				new File("exportOutput"), // directory for output
				false, // isOVA - false means use directory files, not archive
				false, // allowOverwrite 
				true, // hasDigests
				null, // private key
				null, // certificate
				null // progressMonitor
				);
		try {
			exporter.export();
			assert false; // should have failed with OverwriteNotAllowedException
		} catch (Exception e) {
			assert e instanceof OverwriteNotAllowedException;
			System.out.println("Got expected overwrite error: "+e.getMessage());
		}
		System.out.println("Done");
	}
}
