/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.DigestException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;

import com.ibm.adt.ovf.model.utility.OVFCertificateException;
import com.ibm.adt.ovf.model.utility.PackageImporter;


public class ImportPackageExample {

	public static void main(String[] args) {
		ImportPackageExample example = new ImportPackageExample();
		example.run(args);
	}

	private void run(String[] args) {
		if(args.length != 3) {
			System.out.println("Syntax: <input-ova> <output-dir> <overwrite-allowed>");
			return;
		}
		File ovaFile = new File(args[0]);
		FileInputStream fis;
		try {
			fis = new FileInputStream(ovaFile);
			File directory = new File(args[1]); // target output directory
			boolean overwriteAllowed = "true".equalsIgnoreCase(args[2]);
			PackageImporter importer = new PackageImporter(fis, directory, overwriteAllowed, null);
			importer.importPackage();
			Certificate certificate = importer.getCertificate();
			if(certificate != null) {
				System.out.println("Package was unsigned");
			} else {
				System.out.println("Package was signed");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (DigestException e) {
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (CertificateException e) {
			e.printStackTrace();
		} catch (SignatureException e) {
			e.printStackTrace();
		} catch (OVFCertificateException e) {
			e.printStackTrace();
		} 
	}

}
