/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import java.io.IOException;

import org.dmtf.schemas.ovf.envelope.DeploymentOptionSectionType;
import org.dmtf.schemas.ovf.envelope.DiskSectionType;
import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopeFactory;
import org.dmtf.schemas.ovf.envelope.EnvelopePackage;
import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.NetworkSectionType;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeResourceFactoryImpl;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.FeatureMap;

/**
 * <!-- begin-user-doc --> A sample utility for the '<em><b>envelope</b></em>'
 * package. <!-- end-user-doc -->
 * 
 * @generated
 */
public class SubstGroupExample {
	/**
	 * <!-- begin-user-doc --> Load all the argument file paths or URIs as
	 * instances of the model. <!-- end-user-doc -->
	 * 
	 * @param args
	 *            the file paths or URIs.
	 * @generated
	 */
	public static void main(String[] args) {
		// Create a resource set to hold the resources.
		//
		// A ResourceSet is where EMF holds persisted objects. As the name
		// suggests,
		// it is a set of EMF Resources. When you load an OVF file, it will
		// represented
		// as a Resource in a ResourceSet.
		ResourceSet resourceSet = new ResourceSetImpl();

		// Register the appropriate resource factory to handle all file
		// extensions.
		// Since we are running outside of eclipse, we have to tell EMF how to
		// associate files with models (like OVF). The DEFAULT_EXTENSION is "*"
		// so
		// EMF will associate our Envelope stuff with any file.
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION,
						new EnvelopeResourceFactoryImpl());

		// Register the package to ensure it is available during loading.
		// Since we are running outside of eclipse, we need to register our OVF
		// metamodel
		// with the nsURI that identifies it. For OVF, this is
		// "http://schemas.dmtf.org/ovf/envelope/1".
		resourceSet.getPackageRegistry().put(EnvelopePackage.eNS_URI,
				EnvelopePackage.eINSTANCE);

		try {
			// Create a default OVF descriptor

			// Create an empty resource in our resource set. The URI is used to
			// identify
			// the location of the file to be saved or loaded. For standalone
			// applications,
			// you can use URIs like file://.... For this example, we are going
			// to save
			// the OVF file to the system out stream, so it doesn't matter what
			// we use for the
			// URI.
			Resource resource = resourceSet.createResource(URI
					.createURI("http:///My.ovf"));

			// The root element for our OVF metamodel is DocumentRoot. This is a
			// fictitious
			// element that serves as a container for various XML things that
			// don't
			// really belong to the real root element, EnvelopeType.
			// Use the EnvelopeFactory to create instances of OVF objects
			// instead
			// of java constructors -- EMF needs to initialize internal stuff
			// via
			// the factory classes.
			DocumentRoot documentRoot = EnvelopeFactory.eINSTANCE
					.createDocumentRoot();

			// The resource is empty. We need to add the root to the resource.
			// Normally,
			// this will be the only contents for the resource. If we were to
			// add other
			// objects to the resource, they would be saved to the file, which
			// would not
			// be valid for OVF. All the objects that are contained within the
			// root are
			// automatically part of the resource.
			resource.getContents().add(documentRoot);

			// The document root has no Envelope type yet
			EnvelopeType envelope = EnvelopeFactory.eINSTANCE
					.createEnvelopeType();
			documentRoot.setEnvelope(envelope);


			DiskSectionType diskSection = EnvelopeFactory.eINSTANCE
					.createDiskSectionType();

			NetworkSectionType networkSection = EnvelopeFactory.eINSTANCE.
					createNetworkSectionType();
			
			DeploymentOptionSectionType deploymentOptions = 
				EnvelopeFactory.eINSTANCE.createDeploymentOptionSectionType();
			
			// The EnvelopeType.sectionGroup is a list where each element
			// identifies a structural feature with the value.
			// The OVF schema defines Sections as substitution groups and as
			// generic Section types. 
			// a DocumentRoot reference.
			// 
			FeatureMap sectionMap = envelope.getSectionGroup();
			
			// Each substitution group is defined by a documentRoot reference.
			// Add a disk section as a "strongly typed" substitution group
			sectionMap.add(EnvelopePackage.Literals.DOCUMENT_ROOT__DISK_SECTION,
					diskSection);
			
			// Add a network section as a loosely typed element
			sectionMap.add(EnvelopePackage.Literals.ENVELOPE_TYPE__SECTION,
					networkSection);
			
			// There are two sections defined: one generic, one substitution group
			assert(envelope.getSectionGroup().size() == 2);
			
			// The EnvelopeType.section is a feature derived from getSectionGroup()
			assert(envelope.getSection().size() == 2);
			
			// You can also use the derived feature to add new elements
			envelope.getSection().add(deploymentOptions);
			
			// Verify the deployment options were added as a generic section type
			FeatureMap.Entry entry =envelope.getSectionGroup().get(2);
			assert(entry.getEStructuralFeature() == 
				EnvelopePackage.Literals.ENVELOPE_TYPE__SECTION);
			assert(entry.getValue() == deploymentOptions);
			
			// Show the resulting XML 
			resource.save(System.out, null);
		} catch (IOException exception) {
			exception.printStackTrace();
		}
	}

}
