/*******************************************************************************
 * Licensed Materials - Property of IBM
 * 
 * "Restricted Materials of IBM"
 * 
 *  Copyright IBM Corp. 2008 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * $Id:
 *******************************************************************************/
import org.dmtf.schemas.ovf.envelope.AnnotationSectionType;
import org.dmtf.schemas.ovf.envelope.DiskSectionType;
import org.dmtf.schemas.ovf.envelope.EnvelopeFactory;
import org.dmtf.schemas.ovf.envelope.NetworkSectionType;
import org.dmtf.schemas.ovf.envelope.ReferencesType;
import org.dmtf.schemas.ovf.envelope.SectionType;
import org.dmtf.schemas.ovf.envelope.util.EnvelopeSwitch;
import org.eclipse.emf.ecore.EObject;

public class SwitchExample {

	public static void main(String[] args) {
		SwitchExample example = new SwitchExample();
		example.run(args);
	}

	private void run(String[] args) {
		EnvelopeSwitch<String> sw1 = new EnvelopeSwitch<String>() {

			@Override
			public String caseAnnotationSectionType(AnnotationSectionType object) {
				System.out.println("Annotation section found, not handled");
				return null;
			}

			@Override
			public String caseDiskSectionType(DiskSectionType object) {
				System.out.println("Disk section found, handled");
				return "disk";
			}

			@Override
			public String caseSectionType(SectionType object) {
				System.out.println("Generic section, handled");
				return "section";
			}
			
			@Override
			public String defaultCase(EObject object) {
				System.out.println("default case, not handled");
				return null;
			}
		};

		AnnotationSectionType annoSection = EnvelopeFactory.eINSTANCE.createAnnotationSectionType();
		DiskSectionType diskSection = EnvelopeFactory.eINSTANCE.createDiskSectionType();
		NetworkSectionType networkSection = EnvelopeFactory.eINSTANCE.createNetworkSectionType();
		ReferencesType refs = EnvelopeFactory.eINSTANCE.createReferencesType();		
		String result;
		
		System.out.println("-- Annotation Section switch");
		result = sw1.doSwitch(annoSection);
		System.out.println("-- Annotation Section result: "+result);
		assert("section".equals(result));
		System.out.println("\n-- Disk Section switch");
		result = sw1.doSwitch(diskSection);
		System.out.println("-- Disk Section result: "+result);
		assert("disk".equals(result));
		System.out.println("\n-- Network Section switch");
		result = sw1.doSwitch(networkSection);
		System.out.println("-- Network Section result: "+result);
		assert("section".equals(result));
		System.out.println("\n-- References switch");
		result = sw1.doSwitch(refs);
		System.out.println("-- References result: "+result);
		assert(result == null);
	}
}
